
################################################################################
# calnexTranslation.tcl 
#
#	This file contains functions to translate from Paragon-X style commands
#	into REST for Paragon-100G and Paragon-neo.
#
#	This translation is required specifically in the case where files (such as
#	reports, captures, data exports) are expected to end up on the user's local
#	PC. Since the 100G and neo both have browser based UIs, the instrument
#	itself can't write to the local PC. In this case, the files are downloaded
#	by the functions here and written to the local PC.
#
#	The same mechanism may be used for other purposes at some point in the future.
#
#	It is expected that the functions in this file are called from paragon.tcl
#	(not directly).
#
#	The actual functionality for performing the 100G or neo functions is
#	contained in calnexRestModified.tcl. Currently, calnexRestModified.tcl is a
#	specially modified version of that delivered in the instrument - they may
#	be merged in some future release.
#	It is assumed that calnexRestModified.tcl is sourced by paragon.tcl
#	(since this	is not done here).
#
# Copyright (c) Calnex Solutions Ltd 2008 - 2019
#	The contents of this file should not be copied or distributed
#	without permission being granted by Calnex Solutions Ltd.
#
#	All rights reserved.
#
################################################################################
# Translated paragonset commands:
#	Cat Show [neo/100G]: This does nothing for Paragon-neo/100G and so is simply
#		bypassed
#	Cat GenerateReport [neo/100G]: Generates the report and downloads it from the
#		instrument into the folder specified
#	Cat <fileType> [neo/100G]: File load commands are not required for neo/100G
#		and so these are simply bypassed
#
# Translated paragonget commands:
#	Cat <1ppsMetric> OnePpsMiss [X/neo/100G]: The CAT currently does not provide
#		mechanism to return a count of missing 1pps pulses in compatibility
#		mode. This is a new P-X style command to return this count

#-------------------------------------------------------------------------------
# _CatSetTranslation --
#	Parses the incoming command and translates it if necessary.
#
# Arguments:
#	arguments	The command to be parsed along with any parameters
#
# Results:
#	Returns True to indicating that the command has been translated and no
#	further processing is required. Returns False to indicate that the caller
#	should process normally.
#
proc _CatSetTranslation {arguments} {
#	global [namespace current]::__catSlot
	set args [join $arguments]
	set bypass False
	
	if {[lindex $args 0 ] == "Cat"} {
		set isCat True
		set catCommand [lindex $args 1]
		#puts "paragonset Cat Command: $catCommand"
			
		if {$catCommand == "SelectSlot"} {
			# Look for Cat SelectSlot. If found, record the slot. This will be used in later CAT commands
#			set slot [lindex $args 2]
#			if {$slot == "1PPS"} {
#				set slot "1ppsTEAbsolute"
#			}
#			set __catSlot $slot
#			#puts "Cat Slot: $__catSlot"

		} elseif {$catCommand == "Show"} {
			set bypass True
			#_CatShow [lindex $args 2]

		} elseif {$catCommand == "GenerateReport"} {
			set bypass True
			_CatGenerateReport [lindex $args 2] [lindex $args 3]
			
		} elseif {[_IsCatFileCommand $catCommand]} {
			# Ignore P-X launch-with-file related commands
			set bypass True

		}
#		elseif {[_IsCatMetric $catCommand]} {
			# <metrics>
#			set token [lindex $args 2]
			#puts "paragonset $catCommand $token"
			# Need to deal with Export, ExportReducedCsv, ExportReducedTxt, ExportFullCsv, ExportFullTxt
#			if {$token == "Export"} {
				# <metric> Export
				#puts "paragonset calling export"
				#_catEnableMetric $catCommand [lindex $args 3]
				#set bypass True
#			} elseif {$token == "ExportReducedCsv"} {
				# <metric> ExportReducedCsv
				#_catSetAveragingPeriod $__catSlot $catCommand [lindex $args 3]
				#set bypass True
#			}
#		}
		# Need to add Cat <type> Export
		# Need to add Cat SaveAsImage
		# Need to add Cat SaveAsPdf
	}

	return $bypass
}

#-------------------------------------------------------------------------------
# _CatShow -- NOTE: THIS COMMAND NOW DOES NOTHING
#	Starts a new tab in the default browser for the CAT
#
# Arguments:
#	enable	Boolean. True will show the new tab. False is effectively a NOP
#
# Results:
#	None
#
# Notes: This command now does nothing. The problem is that, although we can
#	open a tab in the browser, there appears to be no way to close it. Since
#	Cat Show may be called multiplee times in a test script, this would result
#	in multiple tabs being open in the browser.
#
proc _CatShow {enable} {
	global [namespace current]::__instrumentIP

	if {$enable} {
	#	eval exec [auto_execok start] \"\" [list "http://$__instrumentIP/CAT/#screen=resultsScreen"]
	}
	
	retrun
}

#-------------------------------------------------------------------------------
# _IsCatFileCommand --
# 	There are a number of CAT commands related to launching with different file
#	types. None of these are relevant for Paragon-100G or Paragon-neo
#	This function determines whether the command is one of these file open
#	commands.
#
# Arguments:
#	token	The token following "Cat" in the command
#
# Results:
#	Returns True if the token is a file open/close command and False otherwise.
#
proc _IsCatFileCommand {token} {
	set commands	{Close 1PPS E1T1 2M SyncE Delta1588ModePacketRate \
						1588Sync 1588DelayReq 1588Rtd 1588TimeError \
						1588TCAccuracy 1588PDV NtpClientPdv NtpServerPdv NtpRtd}
	set isCommand False

	set found [lsearch [string tolower $commands] [string tolower $token]]
	if {$found >= 0} {
		set isCommand True
	}
	return $isCommand

}

#-------------------------------------------------------------------------------
# _IsCat1ppsMetricCommand --
# 	Determines whether any given metric is valid for 1pps measurements
#
# Arguments:
#	token	The token following "Cat" in the command
#
# Results:
#	Returns True if the token is a 1pps metric and False otherwise.
#

proc _IsCat1ppsMetricCommand {token} {
	set commands	{TIMEERROR AVERAGEDTE DTELF DTEHF DTEMTIE DTETDEV}
	set isCommand False

	set found [lsearch [string tolower $commands] [string tolower $token]]
	if {$found >= 0} {
		set isCommand True
	}
	return $isCommand
}

#-------------------------------------------------------------------------------
# _IsCatMetricStatsCommand --
# 	Determines whether the command is getting a metric statistic
#
# Arguments:
#	token	The tokens following "Cat" in the command
#
# Results:
#	Returns True if any tokens are stats and False otherwise.
#

proc _IsCatMetricStatisticsCommand {token} {
	set statistics	{Mean cTe Max Min Range PkPk Rate FwdRate RevRate Messages FwdMessages RevMessages}
	set isStat False

	set found [lsearch [string tolower $statistics] [string tolower $token]]
	if {$found >= 0} {
		set isStat True
	}
	return $isStat
}

#-------------------------------------------------------------------------------
# _CatGenerateReport --
# 	The second parameter to Cat GenerateReport is a filename
# 	For CAT with Paragon-X, the filename should be a fully qualified path
# 	This ends up with the report being saved on the user's local PC
# 	For Paragon-100G and Paragon-neo, the instrument has no access to the
# 	user's PC and the report is generated in <IP>/calnex100g/CAT/Reports
# 	So, what we will do here is have the file generated on the instrument
# 	with the specified filename but then copy it to the local PC.
# 	If no path is specified, then we will copy it into the current working directory
#
# Arguments:
#	withCharts	Boolean to indicate whether charets shoudl be included in the report
#	filename	The fully qualified filename for the generated report.
#
# Results:
#	The report will be generated and stored in the specified location. If the
#	supplied filename has no path or has "./" as the path, then the file is
# 	stored in the current working directory
#
proc _CatGenerateReport {withCharts filename} {
	set folder [file dirname $filename]
	set f [file tail $filename]
	calnexCatGenerateReport $withCharts $f $folder
	
	return
}

#-------------------------------------------------------------------------------
# _CatGetTranslation --
#	Parses the incoming command and translates it if necessary.
#
# Arguments:
#	arguments	The command to be parsed along with any parameters
#
# Results:
#	Returns two values:
#	1. Bypass: True to indicate that the command has been translated and no
#	further processing is required. Returns False to indicate that the caller
#	should process normally.
#	2. Value: The response to the get. This is only valid when bypass=true
#
proc _CatGetTranslation {arguments} {
#	global [namespace current]::__catSlot
	set args [join $arguments]
	set bypass False
	set retval "NA"
	set firstArg ""
	
	logDebug "CatGetTransalation: $args"

	if {[lindex $args 0 ] == "Cat"} {
		set isCat True
		set catCommand [lindex $args 1]
		if {[llength $args] > 2} {
			set firstArg [lindex $args 2]
		}
		
		logDebug "Cat Command: $catCommand $firstArg"

		if {[_IsCat1ppsMetricCommand $catCommand]} {
			logDebug "1pps metric command $catCommand $firstArg"
			set stat [lindex $args 2]
			if {$stat == "OnePpsMiss"} {
				set bypass True
				set retval [calnexCatGet1ppsMissingCount]
			}
		} elseif {[_IsCatMetricStatisticsCommand $firstArg]} {
			# We aren't actually doing the get here - just make sure that
			# the CAT says it is done before fetching
			# UPDATE: Don't think we can do this... don't have a per matric status
			#set bypass False
			#calnexCatDone
		}
	}
	logDebug "$bypass $retval"
	return "$bypass $retval"
}

proc _CatWait {} {
	calnexCatDone
}
