################################################################################
# paragon.tcl 
#
#	Tcl interface to Calnex Paragon
#	This file supports Paragon-X, Paragon-100G and Paragon-neo.
#	Not all functions are available for all instruments.
#	See the Paragon Remote Control Manual for details.
#
#	Some functionality for Paragon-100G and Paragon-neo is implemented directly
#	in this wrapper and not in the instrument (for example any command that
#	involves exporting a file to a folder on a local PC). In this case, the
#	additional functionality is implemented in calnexTranslation.tcl which
#	is expected to exist in the same folder as paragon.tcl.
#	calnexTranslation.tcl makes use of calnexRestModifeid.tcl to communicate with
#	Paragon-100G / Paragon-neo. calnexRestModified.tcl also needs to exist in the same
#	folder as paragon.tcl
#	Note: calnexRestModified.tcl has been updated to support the translation
#	functionality. These updates may be integrated into the
#	Paragon-100G / Paragon-neo at some point.
#
#	Logging functionality is implemented in paragonLogging.tcl which is expected
#	to exist in the same folder as paragon.tcl
#
# Copyright (c) Calnex Solutions Ltd 2008 - 2019
#	The contents of this file should not be copied or distributed
#	without permission being granted by Calnex Solutions Ltd.
#
#	All rights reserved.
#
################################################################################
# 27 Jun 2019 [GM]: Added retries in paragonget and paragonset to address
#   timing issues with Paragon-neo and Paragon-100G
# 23 Jan 2019 [GM]: Add logging support (based on 8 Dec 2018 beta version)
#
set revStr "\$Revision: 26110 $"

#-------------------------------------------------------------------------------
# Logging functionality is provided in paragon_logging.tcl
# If this file does not exist in the same folder as paragon.tcl, logging will
# not be possible
# Get the path to paragon.tcl
set [namespace current]::__WrapperFolder [file dirname [info script]]
set [namespace current]::__LoggingFileNotOk [catch {
	source [file join "$__WrapperFolder" "paragonLogging.tcl"]
} __errMsg]

#-------------------------------------------------------------------------------
# Paragon-100G and Paragon-neo translation (required where files need to be exported)
# is supported by calnexRest.tcl and calnexTranslate.tcl
# calnexRest.tcl requires rest and json - check that these are installed
set [namespace current]::__TranslationPackagesNotOk [catch {
	package require rest
	package require json
	package require json::write
} __errMsg]

# If these files do not exist in the same folder as paragon.tcl, then
# translation will not be possible
set [namespace current]::__TranslationFilesNotOk FALSE
if {![file exists [file join "$__WrapperFolder" "calnexRestModified.tcl"]] \
		|| ![file exists [file join "$__WrapperFolder" "calnexTranslation.tcl"]]} {
	set [namespace current]::__TranslationFilesNotOk TRUE
}

set [namespace current]::__TranslationOk TRUE
if {$__TranslationFilesNotOk || $__TranslationPackagesNotOk} {
	set __TranslationOk FALSE
}
################################################################################


# The provided commands are described below. Errors are signaled using the
# TCL error method.

# PRIVATE
# -------

# Interface functions follow below - do not modify these


################################################################################
#
#  Socket handling functions
#
################################################################################


#set __sock ""
#set __lastError ""

set [namespace current]::__sock ""
set [namespace current]::__lastError ""

# On connect, the instrument Idn is parsed to determine the instrument type
set [namespace current]::__instrumentType ""
set [namespace current]::__instrumentIP ""

# The revision number of the wrapper
set [namespace current]::__revNumber [string range $revStr 11 [string length $revStr]-3]


# Log to console
proc logInfo { msg } {
	variable __LoggingFileNotOk
	if {!$__LoggingFileNotOk} {
		wrapperLogDebug "# $msg"
		if {![wrapperLoggingIsConsoleEnabled]} {puts $msg}			
	} else {
	    puts $msg		
	}
}

################################################################################
#
#  Private functions
#
################################################################################


proc guiconnect { {hostname localhost} {port 9000} } {
	global [namespace current]::__sock

    logInfo "connecting to GUI $hostname:$port"
    # Open socket to the given port on the given hostname
    if [catch {
        set __sock [socket $hostname $port]
        fconfigure $__sock -buffering line
    } ] {
        return -code error "Unable to connect to the GUI"
    }
    if { [gets $__sock line] != -1 } {
        logInfo "connected to GUI"
    } else {
        error "Error connecting to GUI"
    } 
}

proc guidisconnect {} {
	variable [namespace current]::__sock

# If we are trying to disconnect and aren't connected then simply ignore the GUI connection status
#    if { [ string compare "" $__sock ] == 0 } {
#        error "Not connected to GUI"
#    }
#    close $__sock
#    set __sock ""
    if { [ string compare "" $__sock ] != 0 } {
        close $__sock
        set __sock ""
    }
}

proc __send { message } {
	variable [namespace current]::__sock
	global [namespace current]::__lastError

    if { [ string compare "" $__sock ] == 0 } {
        error "Not connected to GUI"
    }
    
    # Send the message to the GUI
    #logDebug $message
    puts $__sock $message
    
    # 
    # Wait for response from the GUI
    #     Errors are returned as a string beginning 'ERROR: '
    #     Ready is signalled by '__PARAGON__' prompt
    #     Anything else is considered a response
    # 
    
    set response ""
    set __lastError ""
    
    while { 1 } {
        if { [gets $__sock line] == -1 } {
            error "Socket error"
        }
        
        if {[string first "ERROR: " $line] == 0 } {
            set __lastError "$line"
            #puts "__send, line: $line"
        }
            
		if {[string first "WARNING: " $line] == 0 } {
            set __lastError "$line"
        }
        if {[string compare $line "__PARAGON__"] == 0 } { 
            break
        } else {
			if { $response eq "" } {
			} else {
			  append response "\r\n"
			}
            append response $line
        } 
    }
    
    return $response 
}

proc __chkForError { command } {
	variable [namespace current]::__lastError
    if {[string compare $__lastError ""] != 0 } {
		logDebug "$__lastError (command=\"$command\")"
		error "$__lastError (command=\"$command\")"
	}
}

proc isguiconnected {} {
	variable [namespace current]::__sock
    if { [ string compare "" $__sock ] == 0 } {
        set resp 0
    } else {
        set resp 1
    }
    return $resp
}

proc isconnected { {instrumentIP ""} } {
	global [namespace current]::__sock
	
	logDebug "isconnected"
# Send generates an error if not connected to the GUI. For isconnected, we should simply return false
    if { [isguiconnected] == 0 } {
        set resp 0
    } else {
        set resp [ __send "paragon.remoteprint(paragon.isconnected(\"$instrumentIP\"))" ]
        __chkForError "isconnected"
    }
	logDebug "# Response: $resp"
    return $resp
}

proc remotecontrolon { {ll 1} } {
    __send "paragon.on($ll)"
    __chkForError "remotecontrolon"
}

proc remotecontroloff {} {
    __send "paragon.off()"
    __chkForError "remotecontroloff"
}

proc instrumentconnect { ipAddress {port 9990} } {
    logInfo "connecting to instrument $ipAddress:$port"
    __send "paragon.connect(\"$ipAddress\",\"$port\")"
    __chkForError "instrumentconnect $ipAddress:$port"
}

proc instrumentdisconnect {} {
    __send "paragon.disconnect()"
    __chkForError "instrumentdisconnect"
}

proc _queryerrors {} {
    __send "paragon._queryerrors()"
    __chkForError "queryerrors"
}

proc setoperatingmode { operatingMode } {
    __send "paragon.CSettingParameter(\"__SET____#__OperatingMode__#__$operatingMode\")"
    __chkForError "setoperatingmode $operatingMode"
}


# PUBLIC
# ------

# Get the instrument type from the Idn string
proc getInstrumentType {} {
	set idnString [paragonget Idn]
	set idnParts [split $idnString ,]
	set type [lindex $idnParts 1]
	return $type
}

## Makes a connection to the specified GUI and instrument.
#  @param instrumentIP   required  This is the IP address of the Paragon instrument.
#  @param guihost        optional  This specifies the IP address of the PC that is hosting
#                                  the GUI; if this parameter is not specified then a
#                                  connection to the local machine is assumed.
#  @param instrumentPort optional  This specifies the TCP Port on the Paragon instrument;
#                                  if this parameter is not specified then port 9990 is assumed.
#  @param guiPort        optional  This specifies the remote control TCP Port used by the
#                                  Paragon client application; if this parameter is not
#                                  specified then port 9000 is assumed.
#  @throw error
proc connect { instrumentIP {guihost localhost} {instrumentPort 9990} {guiPort 9000} {ll 1} } {
#    global __lastError
	global [namespace current]::__lastError
	variable [namespace current]::__instrumentType
	global [namespace current]::__instrumentIP
	variable [namespace current]::__revNumber
	variable [namespace current]::__TranslationPackagesNotOk
	variable [namespace current]::__TranslationFilesNotOk
	variable [namespace current]::__WrapperFolder
	
    guiconnect $guihost $guiPort

    set status [ catch {
                     remotecontrolon $ll
                     set connStatus [isconnected]
                     if { $connStatus < 2 } {
                         set connThis [isconnected $instrumentIP]
                         if { $connStatus == 1 && $connThis == 0 } {
                             set currIp [paragonget RmtIpAddress]
                             set __lastError "ERROR: another instrument ($currIp) is already connected"
                             __chkForError "connect"
                         }
                         if { $connThis == 0 } {
                             instrumentconnect $instrumentIP $instrumentPort
                         }
                     }
                 }
               ]
    
    # Force Disconnect
    if { $status != 0 } {
        set err $__lastError
        disconnect
        set __lastError $err
        __chkForError "connect"
    } else {
		set __instrumentType [getInstrumentType]
		set __instrumentIP $guihost
        #logInfo "connected to $__instrumentType"
		logInfo "[paragonget Idn] ($__revNumber)"
#		if {$__instrumentType == "Paragon-100G" || $__instrumentType == "Paragon-Neo"}
			if {!$__TranslationPackagesNotOk && !$__TranslationFilesNotOk} {
				# All good - source the files
				source [file join "$__WrapperFolder" "calnexRestModified.tcl"]
				source [file join "$__WrapperFolder" "calnexTranslation.tcl"]	
				if {$__instrumentType == "Paragon-100G" || $__instrumentType == "Paragon-Neo"} {
					calnexInit $__instrumentIP
				} else {
					pxInit "localhost:1337"
				}
			} else {
				if {$__TranslationFilesNotOk} {
					logInfo "-----"
					logInfo "One or more files cannot be found (calnexTranslation.tcl, calnexRest.tcl)."
					logInfo "These files are needed to download from your instrument to your local PC."
					logInfo "This affects all export operations - the data files will be created but will remain on the instrument"
					logInfo "In addition, you will not be able to read a count of missing 1pps pulse from the CAT"
					logInfo "-----"
				} elseif {$__TranslationPackagesNotOk} {
					logInfo "-----"
					logInfo "One or more required packages (rest, json, json::write) are not installed."
					logInfo "These packages are needed to download from your instrument to your local PC."
					logInfo "This affects all export operations - the data files will be created but will remain on the instrument"
					logInfo "In addition, you will not be able to read a count of missing 1pps pulse from the CAT"
					logInfo "-----"
				}
			}			
		# Curly bracket
    }
}

## Disconnnects the currently connected instrument.
proc disconnect {} {
    catch { instrumentdisconnect }
    catch { remotecontroloff }
    catch { guidisconnect }
    set __sock ""
}

## Starts an ALL byte capture.
#  @throw error
proc startpacketcapture {} {
	logDebug "startpacketcapture"
    __send "paragon.startpacketcapture()"
    __chkForError "startpacketcapture"
}

## Starts timing capture.
#  @throw error
proc starttimingcapture {} {
	logDebug "starttimingcapture"
    __send "paragon.starttimingcapture()"
    __chkForError "starttimingcapture"
}

## Stops current capture.
#  @throw error
proc stopcapture {} {
	logDebug "stopcapture"
    __send "paragon.stopcapture()"
    __chkForError "stopcapture"
}

## Enables the impairment as configured in the settings.
#  @throw error
proc startimpairment {} {
	logDebug "startimpairment"
    __send "paragon.startimpairment()"
    __chkForError "startimpairment"
}

## Disables the impairment as configured in the settings.
#  @throw error
proc stopimpairment {} {
	logDebug "stopimpairment"
    __send "paragon.stopimpairment()"
    __chkForError "stopimpairment"
}

## Starts Time of Day capture.
#  @throw error
proc starttodcapture {} {
	logDebug "starttodcapture"
    __send "paragon.starttodcapture()"
    __chkForError "starttodcapture"
}

## Stops Time Of Day capture.
#  @throw error
proc stoptodcapture {} {
	logDebug "stoptodcapture"
    __send "paragon.stoptodcapture()"
    __chkForError "stoptodcapture"
}

# These use the Lua string functions to convert to Windows-style filename separators



## Saves the displayed captured data to a file.
#  @param filename required  Path and filename for a file on the local PC's
#                            file system. filename must be suffixed with a
#                            valid filename extension:
#                            - ".csv"
#                            - ".cpd"
#  @throw error
proc exportdata { filename } {
	logDebug "exportdata $filename"
    __send "paragon.exportdata(string.gsub(\"$filename\",\"/\",\"\\\\\"))"
    __chkForError "exportdata  $filename"
}

## Loads the captured data from a file.
#  The importdata is equivalent to using the GUI toolbar menu item File>Import. 
#  This is not port specific and in the bi-directional context is not intended
#  to be used for loading impairment data for replay.
#  @param filename required  Path and filename for a file on the local PC's
#                            file system. filename must be suffixed with a
#                            valid filename extension:
#                            - ".csv"
#                            - ".cpd"
#  @throw error
proc importdata { filename } {
	logDebug "importdata $filename"
    __send "paragon.importdata(string.gsub(\"$filename\",\"/\",\"\\\\\"))"
    __chkForError "importdata $filename"
}

## Stores the instrument settings to the file passed in using the filename
#  parameter. Settings are stored in the file system of the machine hosting
#  the GUI. The stored settings file can be recalled using the 'recall' command.
#  If the specified file already exists, then it will be overwritten.
#  @param filename required  Path and filename for a file on the local PC's
#                            file system.
#  @throw error
proc store { filename } {
	logDebug "store $filename"
    __send "saveSettings(string.gsub(\"$filename\",\"/\",\"\\\\\"))"
    __chkForError "store $filename"
}

## Recalls the instrument settings from the file passed in using the filename
#  parameter. Settings are stored in the file system of the machine hosting
#  the GUI. A stored settings file can be created by from the Paragon GUI
#  using Save on the Setup menu.
#  @param filename required  Path and filename for a file on the local PC's
#                            file system.
#  @throw error
proc recall { filename } {
	logDebug "recall $filename"
    __send "loadSettings(string.gsub(\"$filename\",\"/\",\"\\\\\"))"
    __chkForError "recall $filename"
}

################################################################################
# The following routines are used to make the Rst process more robust
# There are occasions where Rst has issues when the instrument is active
# These routines try to make sure that the box is in an idle state before the 
# Rst is sent.
# Note: The Rst command is intercepted in paragonset; anything that is active on
# the instrument is then stopped and then the Rst is passed on.
# Since this file is also used for P100G, a number of the commands may fail
# if any given feature has not yet been implemented. This is avoided by making
# sure that errors are ignored in the appropriate places

##
# Check to see if any wander generation is running
# If so, stop it
proc _stopWander {} {
	#puts "stopWander"
	set isFreqOffsetRunning False
	set tolSingleState  	IDLE
	set tolTableState  		IDLE
	set tolMtieTdevState  	IDLE
	set synceTransientState IDLE
	set transferSingleState IDLE
	set transferTableState 	IDLE
	set transferTdevState 	IDLE
	
	catch {set isFreqOffsetRunning [paragonget WanderGeneration FrequencyOffset Enable]}
	catch {set tolSingleState  [paragonget WanderGeneration Tolerance Single State]}
	catch {set tolTableState  [paragonget WanderGeneration Tolerance Table State]}
	catch {set tolMtieTdevState  [paragonget WanderGeneration Tolerance MtieTdev State]}
	catch {set synceTransientState [paragonget WanderGeneration SyncETransient State]}
	catch {set transferSingleState [paragonget WanderGeneration Transfer Single State]}
	catch {set transferTableState [paragonget WanderGeneration Transfer Table State]}
	catch {set transferTdevState [paragonget WanderGeneration Transfer TDEV State]}
	
	if {$isFreqOffsetRunning} {
		paragonset WanderGeneration FrequencyOffset Enable FALSE
	}
	if {$tolSingleState != "IDLE"} {
		paragonset WanderGeneration Tolerance Single Enable FORCESTOP
	}
	if {$tolTableState != "IDLE"} {
		paragonset WanderGeneration Tolerance Table Enable FORCESTOP
	}
	if {$tolMtieTdevState != "IDLE"} {
		paragonset WanderGeneration Tolerance MtieTdev Enable STOP
	}
	if {$synceTransientState != "IDLE"} {
		paragonset WanderGeneration SyncETransient Enable STOP
	}
	if {$transferSingleState != "IDLE"} {
		catch {paragonset WanderGeneration Transfer Single CalibrateEnable FORCESTOP}
		catch {paragonset WanderGeneration Transfer Single GenerateEnable FORCESTOP}
	}
	if {$transferTableState != "IDLE"} {
		catch {paragonset WanderGeneration Transfer Table CalibrateEnable FORCESTOP}
		catch {paragonset WanderGeneration Transfer Table GenerateEnable FORCESTOP}
	}
	if {$transferTdevState != "IDLE"} {
		paragonset WanderGeneration Transfer TDEV Enable STOP
	}

}

proc _stopPacketGeneration {} {
	#puts "stopPacketGeneration"
	set isPort0Running False
	set isPort1Running False
	
	catch {set isPort0Running [paragonget PacketGeneration #0 Enable]}
	catch {set isPort1Running [paragonget PacketGeneration #1 Enable]}

	if {$isPort0Running} {
		paragonset PacketGeneration #0 Enable FALSE
	}
	if {$isPort1Running} {
		paragonset PacketGeneration #1 Enable FALSE
	}
}

# Check to see if any jitter generation is running
# If so, stop it
proc _stopJitter {} {
	#puts "stopJitter"
	set tolSingleState  	IDLE
	set tolTableState  		IDLE
	set maxTolTableState  	IDLE
	
	catch {set tolSingleState  [paragonget Jitter Tolerance Single State]}
	catch {set tolTableState  [paragonget Jitter Tolerance Table State]}
	catch {set maxTolTableState  [paragonget Jitter MaxTolerable Table State]}
	
	if {$tolSingleState != "IDLE"} {
		paragonset Jitter Tolerance Single Enable FORCESTOP
	}
	if {$tolTableState != "IDLE"} {
		paragonset Jitter Tolerance Table Enable FORCESTOP
	}
	if {$maxTolTableState != "IDLE"} {
		paragonset Jitter MaxTolerable Table Enable FORCESTOP
	}
}

proc _stopMSE {} {
	#puts "stopMSE"
	set isMaster0Running False
	set isMaster1Running False
	set isSlaveRunning 	 False
	
	catch {set isMaster0Running [paragonget MasterSlave Master #0 Enabled]}
	catch {set isMaster1Running [paragonget MasterSlave Master #1 Enabled]}
	catch {set isSlaveRunning [paragonget MasterSlave Slave Enabled]}
	#puts "stopMSE: $isMaster0Running $isMaster1Running $isSlaveRunning"
	
	if {$isMaster0Running} {
		paragonset MasterSlave Master #0 Enabled FALSE
	}
	if {$isMaster1Running} {
		# Looks like the get doesn't return the correct status so ignore errors
		catch {paragonset MasterSlave Master #1 Enabled FALSE}
	}
	if {$isSlaveRunning} {
		catch {paragonset MasterSlave Slave Enabled FALSE}
	}
}

proc _stopCapture {} {
	#puts "stopCapture"
	#set retry 0
	stopcapture
	#while {[paragonget InstrumentStatus Capture IsRunning] == TRUE && $retry < 6} {
	#	after 500
	#	stopcapture
	#	incr retry
	#}
}

proc _stopImpairment {} {
	#puts "stopImpairment"
	catch {stopimpairment}
}

## Before executing Rst, make sure that the instrument is idle
# •	If a capture is in progress then we not idle.
# •	If Thru mode is active we are not idle.
# •	If import-export is active we are not idle.
#		We can't check for this and so we ignore it - unlikely to happen
# •	If MSE is active we are not idle.
# •	If impairments are running we are not idle.
# •	If wander generation is active we are not idle.
# •	If Jitter generation is active we are not idle.
# •	If Packet Generation is active we are not idle.
proc _doResetPreparation {} {
	#puts "doResetPreparation"
	
	# Stop anything that is running
	_stopCapture
	_stopImpairment
	_stopWander
	_stopPacketGeneration
	_stopJitter
	_stopMSE
	
	#paragonset SimulMeasImpairMode MEASUREONLY
	#paragonset OperatingMode SERVICES
	#paragonset TxRxMode TRUE	
}

proc _RstTranslation {arguments} {
	set args [join $arguments]

	# We want bypass set to false.
	# Once we are done with the other reset actions, we want to execute the Rst normally
	set bypass False

	if {[lindex $args 0 ] == "Rst"} {
		#puts "rstTranslation $args"
		_doResetPreparation
	}
	return $bypass
}

## Set an instrument setting. Most of the instrument's settings are controlled
#  using by this command. (Refer to the remote control manual for the complete list)
#  @param args   required  Setting to set and its value (e.g: paragonset Physical LineRate 1GBE).
#  @throw error
proc paragonset { args } {
	variable [namespace current]::__instrumentType
	variable [namespace current]::__TranslationOk

	logDebug "paragonset $args"
	
	set isBypass False
    set wireCmd "paragon.CSettingParameter(\"__SET__"

	# Parse the incoming command such that specific actions can be taken for specific commands
	# In most cases, when a command is found, it will bypass further processing
	if {$__instrumentType == "Paragon-X"} {
		set isBypass [_RstTranslation $args]
	}
	if {$__instrumentType == "Paragon-100G" || $__instrumentType == "Paragon-Neo"} {
		if {$__TranslationOk} {set isBypass [_CatSetTranslation $args]}
	}

	# Only send the command is it hasn't already been dealt with
	if {$isBypass == False} {
		foreach arg $args {
			# Indexed: 1st char is '#'
			if {[string first \# $arg] == 0} {
				set idx [string range $arg 1 end]
				set arg "__IDX____#__"
				append arg $idx
			}
			append wireCmd "__#__" $arg
		}
		append wireCmd "\")"

   		# Some paragonset Cat commands may return an error
		# while in live mode. This is caused by the rolling re-calculation.
		# So, make two further attempts to retry if this happens
        # This has been seen with Cat SelectSlot and Cat <metric> Enable
        # The errors are "Undefined command" and "Invalid parameter"
        set retries 1
       	if {[lindex $args 0 ] == "Cat"} {
            set retries 100
        }
		for {set retry 1} {$retry <= $retries} {incr retry} {
		    set resp [__send $wireCmd]
            # Look for "Metric is not calculated"; if present, retry
            # If not, call chkForError
            # If at the end of retries, call chkForError
            set found false
            if {[string first "Undefined command" $resp] != -1 ||
                [string first "Invalid parameter" $resp] != -1} {
                set found true
            }
            if {!$found || $retry == $retries} {
                __chkForError "paragonget $args"                
                break                
            }

            logDebug "RETRY---> paragonset $args --- retry $retry: $resp"
            #puts "RETRY---> paragonset $args --- retry $retry: $resp"
            after 1000
		}
        

		#__send $wireCmd
		#__chkForError "paragonset $args"
	}
}

## Query an instrument setting. Most of the instrument's settings are read
#  using by this command. (Refer to the remote control manual for the complete list)
#  @param args required  Setting to query (e.g: paragonget Physical LineRate).
#  @return Setting value.
#  @throw error
proc paragonget { args } {
	variable [namespace current]::__instrumentType
	variable [namespace current]::__TranslationOk

    set wireCmd "paragon.CSettingParameter(\"__GET__"

	set isBypass False
	logDebug "paragonget $args"

	# Parse the incoming command such that specific actions can be taken for specific commands
	# In most cases, when a command is found, it will bypass further processing
#	if {$__instrumentType == "Paragon-X"} 
		# Nothing to translate
#	
#	if {$__instrumentType == "Paragon-100G" || $__instrumentType == "Paragon-Neo"} 
		if {$__TranslationOk} {
			if {[lindex $args 0 ] == "Cat"} {
				set response [_CatGetTranslation $args]
				set isBypass [lindex  $response 0]
				set resp [lindex $response 1]
			}
		}
	# Curly bracket

	# Only send the command is it hasn't already been dealt with
	if {$isBypass == False} {	
	    foreach arg $args {
	        # Indexed: 1st char is '#'
	        if {[string first \# $arg] == 0} {
	            set idx [string range $arg 1 end]
	            set arg "__IDX____#__"
	            append arg $idx
	        }
	        append wireCmd "__#__" $arg
	    }
	    append wireCmd "\")"
    
		# paragonget Cat <metric> <stat> may return "Metric is not calculated"
		# while in live mode. This is caused by the rolling re-calculation.
		# So, make two further attempts to retry if this happens
        set retries 100
		for {set retry 1} {$retry <= $retries} {incr retry} {
		    set resp [__send $wireCmd]
            # Look for "Metric is not calculated"; if present, retry
            # If not, call chkForError
            # If at the end of retries, call chkForError
            set found false
            if {[string first "Metric is not calculated" $resp] != -1 ||
                [string first "Type of metric doesn't exist" $resp] != -1} {
                set found true
            }
            if {!$found || $retry == $retries} {
                __chkForError "paragonget $args"

    		    set resp [ __send "print(paragon.CSettingParameter_resp())" ]
    		    __chkForError "paragonget $args"

        	    # Ignore the type ID character at the start of the response. Just
        	    # return the data as a string
        	    set retval [string range $resp 1 end]
                
                break                
            }

            logDebug "RETRY---> paragonget $args --- retry $retry: $resp"
            #puts "RETRY---> paragonget $args --- retry $retry: $resp"
            after 1000
		}
	}
	
	logDebug "# paragonget response: $retval"
    return $retval
}

# note that a change in "Wat Status" command causes a change in the implementation of tcl command  wait.

## Wait for the completion of the current CAT operation. 
#  If a script is waiting for previous open operation or waiting for end of
#  previous calculation then this command should be always used. It is
#  recommended to call this after the calls below otherwise there is
#  possibility to get settings conflict error.<br>
#  @throw Exception
proc waitforcat {} {
	variable [namespace current]::__instrumentType
	variable [namespace current]::__TranslationOk

	if {$__instrumentType == "Paragon-100G" || $__instrumentType == "Paragon-Neo"} {
		if {$__TranslationOk} {_CatWait}
	} else {	
		set ok [ paragonget "Wat" "Status"]
		set log 0
		while { $ok == 1 } {
			if {$log == 0} {
				set log 1
			}
	        after 1000
			set ok [paragonget "Wat" "Status"]
		}
	#logInfo "Wait operation completed"
	}
}

proc waitForCat {} {
	waitforcat
}

# These use the Lua string functions to convert to Windows-style filename separators

## Loads the capture data for replay. It will load files specifically to replay
#  delay profiles against defined message types on defined ports.
#  The loaded file will be replayed against incoming traffic on the given port.
#  The importimpairmentdata command is equivalent to using the "Import-Port1" and
#  "Import-Port2" buttons in the Add Impairments and Delay window
#  @param port     required  The Ethernet port (1 or 2).
#  @param filename required  Path to a capture file on the local PC's
#                            file system.
#  @throw error
proc importimpairmentdata { port filename } {
	logDebug "importimpairmentdata $port $filename"
    __send "paragon.importimpairmentdata(\"$port\",string.gsub(\"$filename\",\"/\",\"\\\\\"))"
    __chkForError "importimpairmentdata $port $filename"
}

## Select the SONET STS1 tributary to impair.
#  @param trib required  STS1 tributary number.
#  @throw error
proc setsonetowsts1trib { trib } {
    __send "paragon.CSettingParameter(\"__SET____#___SonetSts1Trib__#__$trib\")"
    __chkForError "setsonetowsts1trib $trib"
}

## Select the SONET VT1.5 tributary to impair.
#  @param trib required  VT1.5 tributary number.
#  @throw error
proc setsonetowvttrib { trib } {
    __send "paragon.CSettingParameter(\"__SET____#___SonetVtTrib__#__$trib\")"
    __chkForError "setsonetowvttrib $trib"
}

## Set the SONET delay
#  @param delay required  Delay in uS.
#  @throw error
proc setsonetdelay { delay } {
    __send "paragon.CSettingParameter(\"__SET____#___SonetDelay__#__$delay\")"
    __chkForError "setsonetdelay $delay"
}

# Wrapper logging ##############################################################
# Notes: These stubs are required to cover the situation where
#	paragonLogging.tcl is not found.
#-------------------------------------------------------------------------------
# paragonWrapperLogging --
#	Enable or disable logging to file. If logging is enabled, log messages may
#	or may not be also sent to the console
#
# Arguments:
#	enable		Boolean. Enable or disable logging to file
#	toConsole	Boolean. Enable or disable logging to the console
#
# Results:
#	When logging is enabled, dated log files will be written to a folder
#	in the current working directory in a folder called ParagonWrapperLogging.
#
proc paragonWrapperLogging { {enable TRUE} {toConsole FALSE} } {
	variable __LoggingFileNotOk
	if {!$__LoggingFileNotOk} {
		wrapperLogging $enable $toConsole
	} else {
		logInfo "Wrapper logging cannot be enabled - paragonLogging.tcl not found"
	}
	return
}

#-------------------------------------------------------------------------------
# logDebug --
#	Write a log message
#
# Arguments:
#	msg		String.
#
# Results:
#	msg is written to the current log file.
#
proc logDebug { msg } {
	variable __LoggingFileNotOk
	if {!$__LoggingFileNotOk} {
		wrapperLogDebug $msg
	}
	return
}
###############################################################################
