################################################################################
# paragonLogging.tcl 
#
#	This file contains functions to support logging of commands processed by
#	paragon.tcl.
#	Dated log files are created in the "ParagonWrapperLogging" folder in the
# 	current working directory.
#
# Copyright (c) Calnex Solutions Ltd 2008 - 2019
#	The contents of this file should not be copied or distributed
#	without permission being granted by Calnex Solutions Ltd.
#
#	All rights reserved.
#
################################################################################

# Dictionary to hold command logging parameters
set [namespace current]::__WrapperLoggingParameters \
		{enabled FALSE console FALSE logFileFid ""}

# Get the current time (private)
proc _CurrentDateTime {} {
	set systemTime [clock seconds]
	set d [clock format $systemTime -format "%Y-%m-%d"]
	set t [clock format $systemTime -format %H-%M-%S]
	return "${d}_${t}"
}

# Open log file (private)	
proc _OpenLogFile {folder filename} {	
	if {![file exists $folder]} {
		file mkdir $folder
	}
	set fid [open [file join "${folder}" "${filename}"] "w"]
	return $fid
}

# Control command logging
# Parameters:
#	enable: TRUE to enable logging; FALSE to disable it.
#	toConsole: TRUE to send logging to the console (as well as to file); FALSE to send only to file
# Description:
# 	When logging is enabled, all commands are written to the log file (and optionally the console)
# 	The log file is stored in a folder called "paragonWrapperLogging".
# 	This folder is created in the same folder in which the calling script is being run.
# 	The log file is stored in a text file called "wrapperLogging_<DateTime>.txt".
# 	Although the log file is created as a text file, it is intended to be able to be used as a tcl file.
# 	All log contents that cannot be run as Tcl are written as Tcl comments.
# Usage:
#	Call paragonWrapperLogging TRUE at the point in our script where you want to log commands
#	Call paragonWrapperLogging FALSE when you no longer want to log commands.
#	Note: The log file will only be closed when either the calling script terminates or
#	paragonWrapperLogging FALSE is called.
proc wrapperLogging { {enable TRUE} {toConsole FALSE} } {
	variable [namespace current]::__WrapperLoggingParameters

	if {$enable == TRUE} {
		# Set global logging parameters
		dict set __WrapperLoggingParameters "enabled" TRUE
		dict set __WrapperLoggingParameters "console" $toConsole

		# Open log file in logging folder
		set fid [_OpenLogFile "./ParagonWrapperLogging" "wrapperLogging_[_CurrentDateTime].txt"]
		dict set __WrapperLoggingParameters "logFileFid" $fid
		
		puts $fid "# Logging started: [_CurrentDateTime]"
		puts $fid "# Called from: [info script]"
		
	} elseif {$enable == FALSE} {
		# Set global logging parameters
		dict set __WrapperLoggingParameters "enabled" FALSE
		dict set __WrapperLoggingParameters "console" FALSE

		# Flush and close log file
		set fid [dict get $__WrapperLoggingParameters "logFileFid"]
		flush $fid
		flush stdout
		close $fid
		
	} else {
		error "wrapperLogging: first parameter (enable) must be TRUE or FALSE"
	}
}

# Log to file and / or console based on how logging was intialised
proc wrapperLogDebug {msg} {
	variable [namespace current]::__WrapperLoggingParameters
	
	set enabled [dict get $__WrapperLoggingParameters "enabled"]
	set console [dict get $__WrapperLoggingParameters "console"]
	set fid [dict get $__WrapperLoggingParameters "logFileFid"]

	if {$enabled == TRUE} {
		puts $fid "# [_CurrentDateTime]"
		#getCallStack
		puts $fid $msg
		flush $fid
	}
	if {$console == TRUE} {
		puts $msg
		flush stdout
	}
}

proc wrapperLoggingIsConsoleEnabled {} {
	variable [namespace current]::__WrapperLoggingParameters
	return [dict get $__WrapperLoggingParameters "enabled"]	
}
